@@ -33,7 +33,7 @@ class DivideMax(nn.Module):
         self.dim = dim
 
     def forward(self, x):
-        maxes = x.amax(dim = self.dim, keepdim = True)
+        maxes = x.amax(dim = self.dim, keepdim = True).detach()
         return x / maxes
 
 # https://arxiv.org/abs/2103.17239
@@ -56,13 +56,16 @@ class LayerScale(nn.Module):
 # layer norm
 
 class PreNorm(nn.Module):
-    def __init__(self, dim, fn):
+    def __init__(self, dim, fn, sandwich = False):
         super().__init__()
         self.norm = nn.LayerNorm(dim)
+        self.norm_out = nn.LayerNorm(dim) if sandwich else nn.Identity()
         self.fn = fn
 
     def forward(self, x, **kwargs):
-        return self.fn(self.norm(x), **kwargs)
+        x = self.norm(x)
+        x = self.fn(x, **kwargs)
+        return self.norm_out(x)
 
 # feed forward
 
@@ -100,6 +103,11 @@ class PreShiftToken(nn.Module):
         text_len = seq_len - img_seq_len + 1
         padding = seq_len - n + 1
 
+        # if sequence is shorter than the text length, no image tokens to shift
+
+        if n < text_len:
+            return self.fn(x, **kwargs)
+
         # get text and image tokens
 
         x_text, x_img = x[:, :text_len], x[:, text_len:]
@@ -145,7 +153,8 @@ class Transformer(nn.Module):
         image_fmap_size = None,
         sparse_attn = False,
         stable = False,
-        shift_tokens = True,
+        sandwich_norm = False,
+        shift_tokens = False,
         rotary_emb = True
     ):
         super().__init__()
@@ -183,8 +192,8 @@ class Transformer(nn.Module):
                 attn, ff = map(lambda t: PreShiftToken(t, image_size = image_fmap_size, seq_len = seq_len), (attn, ff))
 
             layers.append(nn.ModuleList([
-                LayerScale(dim, ind + 1, PreNorm(dim, attn)),
-                LayerScale(dim, ind + 1, PreNorm(dim, ff))
+                LayerScale(dim, ind + 1, PreNorm(dim, attn, sandwich = sandwich_norm)),
+                LayerScale(dim, ind + 1, PreNorm(dim, ff, sandwich = sandwich_norm))
             ]))
 
         execute_type = ReversibleSequence if reversible else SequentialSequence
@@ -219,7 +228,7 @@ class Transformer(nn.Module):
             img_freqs = torch.cat((text_axial_freqs, img_freqs), dim = 0)
 
             pos_emb = torch.cat((text_freqs, img_freqs), dim = -1)
-            pos_emb = rearrange(pos_emb[:-1], 'n d -> () () n d')
+            pos_emb = rearrange(pos_emb, 'n d -> () n d')
 
         self.register_buffer('pos_emb', pos_emb)
 
